//+------------------------------------------------------------------+
//|                                       Exp_AdaptiveCyberCycle.mq5 |
//|                             Copyright  2012,   Nikolay Kositsin | 
//|                              Khabarovsk,   farria@mail.redcom.ru | 
//+------------------------------------------------------------------+
#property copyright "Copyright  2012, Nikolay Kositsin"
#property link      "farria@mail.redcom.ru"
#property version   "1.00"
//+----------------------------------------------+
//|                        |
//+----------------------------------------------+
enum Mode_ // 
  {
   AdaptiveCyberCycle=1,//Adaptive Cyber Cycle
   AdaptiveCGOscillator,//Adaptive CG Oscillator
   AdaptiveRVI          //Adaptive RVI
  };
//+----------------------------------------------+
//|            |
//+----------------------------------------------+
input double MM=-0.1;             //      ,   -  
input int    StopLoss_=1000;      //  
input int    TakeProfit_=2000;    //  
input int    Deviation_=10;       //.    
input bool   BuyPosOpen=true;     //    
input bool   SellPosOpen=true;    //    
input bool   BuyPosClose=true;     //    
input bool   SellPosClose=true;    //    
//+----------------------------------------------+
//|    CyberCycle      |
//+----------------------------------------------+
input ENUM_TIMEFRAMES InpInd_Timeframe=PERIOD_H4; // 
input Mode_ Mode=AdaptiveCyberCycle;//  
input double Alpha=0.07;//  
input uint SignalBar=1;//     
//+----------------------------------------------+
//----          
int TimeShiftSec;
//----      
int InpInd_Handle;
//----      
int min_rates_total;
//+------------------------------------------------------------------+
//                                                  | 
//+------------------------------------------------------------------+
#include <TradeAlgorithms.mqh>
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
//----
   switch(Mode)
     {
      case AdaptiveCyberCycle:
         //----    AdaptiveCyberCycle
         InpInd_Handle=iCustom(Symbol(),InpInd_Timeframe,"AdaptiveCyberCycle",Alpha,0);
         if(InpInd_Handle==INVALID_HANDLE) Print("      AdaptiveCyberCycle");

         break;

      case AdaptiveCGOscillator:
         //----    AdaptiveCGOscillator
         InpInd_Handle=iCustom(Symbol(),InpInd_Timeframe,"AdaptiveCGOscillator",Alpha,0);
         if(InpInd_Handle==INVALID_HANDLE) Print("      AdaptiveCGOscillator");
         break;

      case AdaptiveRVI:
         //----    AdaptiveRVI
         InpInd_Handle=iCustom(Symbol(),InpInd_Timeframe,"AdaptiveRVI",Alpha,0);
         if(InpInd_Handle==INVALID_HANDLE) Print("      AdaptiveRVI");
         break;
     }

//----          
   TimeShiftSec=PeriodSeconds(InpInd_Timeframe);

//----     
   min_rates_total=int(8+3+SignalBar);
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//----
   GlobalVariableDel_(Symbol());
//----
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
//----       
   if(BarsCalculated(InpInd_Handle)<min_rates_total) return;

//----       IsNewBar()  SeriesInfoInteger()  
   LoadHistory(TimeCurrent()-PeriodSeconds(InpInd_Timeframe)-1,Symbol(),InpInd_Timeframe);

//----   
   double Value[2],SigValue[2];
//----   
   static bool Recount=true;
   static bool BUY_Open=false,BUY_Close=false;
   static bool SELL_Open=false,SELL_Close=false;
   static datetime UpSignalTime,DnSignalTime;
   static CIsNewBar NB;

//+----------------------------------------------+
//|                  |
//+----------------------------------------------+
   if(!SignalBar || NB.IsNewBar(Symbol(),InpInd_Timeframe) || Recount) //     
     {
      //----   
      BUY_Open=false;
      SELL_Open=false;
      BUY_Close=false;
      SELL_Close=false;
      Recount=false;

      //----      
      if(CopyBuffer(InpInd_Handle,MAIN_LINE,SignalBar,2,Value)<=0) {Recount=true; return;}
      if(CopyBuffer(InpInd_Handle,SIGNAL_LINE,SignalBar,2,SigValue)<=0) {Recount=true; return;}

      //----    
      if(Value[1]>SigValue[1])
        {
         if(BuyPosOpen && Value[0]<=SigValue[0]) BUY_Open=true;
         if(SellPosClose) SELL_Close=true;
         UpSignalTime=datetime(SeriesInfoInteger(Symbol(),InpInd_Timeframe,SERIES_LASTBAR_DATE))+TimeShiftSec;
        }

      //----    
      if(Value[1]<SigValue[1])
        {
         if(SellPosOpen && Value[0]>=SigValue[0]) SELL_Open=true;
         if(BuyPosClose) BUY_Close=true;
         DnSignalTime=datetime(SeriesInfoInteger(Symbol(),InpInd_Timeframe,SERIES_LASTBAR_DATE))+TimeShiftSec;
        }
     }

//+----------------------------------------------+
//|                              |
//+----------------------------------------------+
//----  
   BuyPositionClose(BUY_Close,Symbol(),Deviation_);

//----     
   SellPositionClose(SELL_Close,Symbol(),Deviation_);

//----  
   BuyPositionOpen(BUY_Open,Symbol(),UpSignalTime,MM,0,Deviation_,StopLoss_,TakeProfit_);

//----  
   SellPositionOpen(SELL_Open,Symbol(),DnSignalTime,MM,0,Deviation_,StopLoss_,TakeProfit_);
//----
  }
//+------------------------------------------------------------------+
